@@ -1,11 +1,9 @@
 import logging
-import math
 import os
 import re
 
 from classy_imaginary.schema import SafetyMode
 from classy_imaginary.model_manager import get_diffusion_model
-from classy_imaginary.img_utils import add_caption_to_image
 
 logger = logging.getLogger(__name__)
 
@@ -23,15 +21,15 @@ _most_recent_result = None
 
 
 def imagine_image_files(
-        prompts,
-        outdir,
-        precision="autocast",
-        record_step_images=False,
-        output_file_extension="jpg",
-        print_caption=False,
-        make_gif=False,
-        make_compare_gif=False,
-        return_filename_type="generated",
+    prompts,
+    outdir,
+    precision="autocast",
+    record_step_images=False,
+    output_file_extension="jpg",
+    print_caption=False,
+    make_gif=False,
+    make_compare_gif=False,
+    return_filename_type="generated",
 ):
     from PIL import ImageDraw
 
@@ -61,10 +59,10 @@ def imagine_image_files(
             p.collect_progress_latents = True
     result_filenames = []
     for result in imagine(
-            prompts,
-            precision=precision,
-            debug_img_callback=_record_step if record_step_images else None,
-            add_caption=print_caption,
+        prompts,
+        precision=precision,
+        debug_img_callback=_record_step if record_step_images else None,
+        add_caption=print_caption,
     ):
         prompt = result.prompt
         if prompt.is_intermediate:
@@ -131,16 +129,16 @@ def imagine_image_files(
 
 
 def imagine(
-        prompts,
-        precision="autocast",
-        debug_img_callback=None,
-        progress_img_callback=None,
-        progress_img_interval_steps=3,
-        progress_img_interval_min_s=0.1,
-        half_mode=None,
-        add_caption=False,
-        unsafe_retry_count=1,
-        nsfw_filter=False,
+    prompts,
+    precision="autocast",
+    debug_img_callback=None,
+    progress_img_callback=None,
+    progress_img_interval_steps=3,
+    progress_img_interval_min_s=0.1,
+    half_mode=None,
+    add_caption=False,
+    unsafe_retry_count=1,
+    nsfw_filter=False,
 ):
     import torch.nn
 
@@ -164,7 +162,7 @@ def imagine(
         logger.info("Running in CPU mode. it's gonna be slooooooow.")
 
     with torch.no_grad(), platform_appropriate_autocast(
-            precision
+        precision
     ), fix_torch_nn_layer_norm(), fix_torch_group_norm():
         for i, prompt in enumerate(prompts):
             logger.info(
@@ -173,10 +171,10 @@ def imagine(
             for attempt in range(0, unsafe_retry_count + 1):
                 if attempt > 0:
                     prompt.seed += 100_000_000 + attempt
-
                 model = get_diffusion_model(
                     weights_location=prompt.model,
                     config_path=prompt.model_config_path,
+                    control_weights_location=prompt.control_mode,
                     half_mode=half_mode,
                     for_inpainting=prompt.mask_image or prompt.mask_prompt or prompt.outpaint,
                 )
@@ -204,7 +202,7 @@ def imagine(
                     nsfw_filter=nsfw_filter,
                     safety_feature_extractor=safety_feature_extractor,
                     safety_checker=safety_checker,
-                    codformer_model=codeformer_net_model,
+                    codeformer_model=codeformer_net_model,
                     debug_img_callback=debug_img_callback,
                     progress_img_callback=progress_img_callback,
                     progress_img_interval_steps=progress_img_interval_steps,
@@ -214,7 +212,7 @@ def imagine(
                 if not result.safety_score.is_filtered:
                     break
                 if attempt < unsafe_retry_count:
-                    logger.info("   Image was unsafe, retrying with new seed...")
+                    logger.info("    Image was unsafe, retrying with new seed...")
 
             yield result
 
@@ -225,12 +223,13 @@ def _generate_single_image(
         nsfw_filter: bool = False,
         safety_feature_extractor=None,
         safety_checker=None,
-        codformer_model=None,
+        codeformer_model=None,
         debug_img_callback=None,
         progress_img_callback=None,
         progress_img_interval_steps=3,
         progress_img_interval_min_s=0.1,
         add_caption=False,
+        suppress_inpaint=False,
         return_latent=False,
 ):
     import torch.nn
@@ -241,21 +240,26 @@ def _generate_single_image(
     from classy_imaginary.enhancers.describe_image_blip import generate_caption
     from classy_imaginary.enhancers.face_restoration_codeformer import enhance_faces
     from classy_imaginary.enhancers.upscale_realesrgan import upscale_image
-    from classy_imaginary.img_utils import (pillow_fit_image_within,
+    from classy_imaginary.img_utils import (
+        add_caption_to_image,
+        pillow_fit_image_within,
         pillow_img_to_torch_image,
         pillow_mask_to_latent_mask,
-        torch_img_to_pillow_img)
+        torch_img_to_pillow_img,
+    )
     from classy_imaginary.log_utils import (
         ImageLoggingContext,
         log_conditioning,
         log_img,
         log_latent,
     )
+    from classy_imaginary.model_manager import (
+        get_model_default_image_size,
+    )
     from classy_imaginary.modules.midas.api import torch_image_to_depth_map
     from classy_imaginary.outpaint import outpaint_arg_str_parse, prepare_image_for_outpaint
     from classy_imaginary.safety import create_safety_score
     from classy_imaginary.samplers import SAMPLER_LOOKUP
-    from classy_imaginary.samplers.base import NoiseSchedule, noise_an_image
     from classy_imaginary.samplers.editing import CFGEditingDenoiser
     from classy_imaginary.schema import ImaginePrompt, ImagineResult
     from classy_imaginary.utils import get_device, randn_seeded
@@ -280,15 +284,15 @@ def _generate_single_image(
         progress_latents.append(latents)
 
     with ImageLoggingContext(
-            prompt=prompt,
-            model=model,
-            debug_img_callback=debug_img_callback,
-            progress_img_callback=progress_img_callback,
-            progress_img_interval_steps=progress_img_interval_steps,
-            progress_img_interval_min_s=progress_img_interval_min_s,
-            progress_latent_callback=latent_logger
-            if prompt.collect_progress_latents
-            else None,
+        prompt=prompt,
+        model=model,
+        debug_img_callback=debug_img_callback,
+        progress_img_callback=progress_img_callback,
+        progress_img_interval_steps=progress_img_interval_steps,
+        progress_img_interval_min_s=progress_img_interval_min_s,
+        progress_latent_callback=latent_logger
+        if prompt.collect_progress_latents
+        else None,
     ) as lc:
         seed_everything(prompt.seed)
 
@@ -313,7 +317,7 @@ def _generate_single_image(
         SamplerCls = SAMPLER_LOOKUP[prompt.sampler_type.lower()]
         sampler = SamplerCls(model)
         mask_latent = mask_image = mask_image_orig = mask_grayscale = None
-        t_enc = init_latent = init_latent_noised = control_image = None
+        t_enc = init_latent = control_image = None
         starting_image = None
         denoiser_cls = None
 
@@ -326,8 +330,9 @@ def _generate_single_image(
         if prompt.init_image:
             starting_image = prompt.init_image
             generation_strength = 1 - prompt.init_image_strength
-            if model.cond_stage_key == "edit":
-                t_enc = prompt.steps
+
+            if model.cond_stage_key == "edit" or generation_strength >= 1:
+                t_enc = None
             else:
                 t_enc = int(prompt.steps * generation_strength)
 
@@ -382,99 +387,107 @@ def _generate_single_image(
             )
             # noise = noise[:, :, : init_latent.shape[2], : init_latent.shape[3]]
 
-            schedule = NoiseSchedule(
-                model_num_timesteps=model.num_timesteps,
-                ddim_num_steps=prompt.steps,
-                model_alphas_cumprod=model.alphas_cumprod,
-                ddim_discretize="uniform",
+            # schedule = NoiseSchedule(
+            #     model_num_timesteps=model.num_timesteps,
+            #     ddim_num_steps=prompt.steps,
+            #     model_alphas_cumprod=model.alphas_cumprod,
+            #     ddim_discretize="uniform",
+            # )
+            # if generation_strength >= 1:
+            #     # prompt strength gets converted to time encodings,
+            #     # which means you can't get to true 0 without this hack
+            #     # (or setting steps=1000)
+            #     init_latent_noised = noise
+            # else:
+            #     init_latent_noised = noise_an_image(
+            #         init_latent,
+            #         torch.tensor([t_enc - 1]).to(get_device()),
+            #         schedule=schedule,
+            #         noise=noise,
+            #     )
+
+        if hasattr(model, "depth_stage_key"):
+            # depth model
+            depth_t = torch_image_to_depth_map(init_image_t)
+            depth_latent = torch.nn.functional.interpolate(
+                depth_t,
+                size=shape[2:],
+                mode="bicubic",
+                align_corners=False,
             )
-            if generation_strength >= 1:
-                # prompt strength gets converted to time encodings,
-                # which means you can't get to true 0 without this hack
-                # (or setting steps=1000)
-                init_latent_noised = noise
-            else:
-                init_latent_noised = noise_an_image(
-                    init_latent,
-                    torch.tensor([t_enc - 1]).to(get_device()),
-                    schedule=schedule,
-                    noise=noise,
-                )
+            result_images["depth_image"] = depth_t
+            c_cat.append(depth_latent)
 
-            if hasattr(model, "depth_stage_key"):
-                # depth model
-                depth_t = torch_image_to_depth_map(init_image_t)
-                depth_latent = torch.nn.functional.interpolate(
-                    depth_t,
-                    size=shape[2:],
-                    mode="bicubic",
-                    align_corners=False,
-                )
-                result_images["depth_image"] = depth_t
-                c_cat.append(depth_latent)
+        elif is_controlnet_model:
+            from classy_imaginary.img_processors.control_modes import CONTROL_MODES
 
-            elif is_controlnet_model and starting_image:
-                from classy_imaginary.img_processors.control_modes import CONTROL_MODES
+            if prompt.control_image_raw is not None:
+                control_image = prompt.control_image_raw
+            elif prompt.control_image is not None:
+                control_image = prompt.control_image
+            control_image = control_image.convert("RGB")
+            log_img(control_image, "control_image_input")
+            control_image_input = pillow_fit_image_within(
+                control_image,
+                max_height=prompt.height,
+                max_width=prompt.width,
+            )
+            control_image_input_t = pillow_img_to_torch_image(control_image_input)
+            control_image_input_t = control_image_input_t.to(get_device())
 
-                control_image_input = pillow_fit_image_within(
-                    prompt.control_image,
-                    max_height=prompt.height,
-                    max_width=prompt.width,
-                )
-                control_image_input_t = pillow_img_to_torch_image(control_image_input)
-                control_image_input_t = control_image_input_t.to(get_device())
-
-                control_image = CONTROL_MODES[prompt.control_mode](
+            if prompt.control_image_raw is None:
+                control_image_t = CONTROL_MODES[prompt.control_mode](
                     control_image_input_t
                 )
-                if len(control_image.shape) == 3:
-                    raise RuntimeError("Control image must be 4D")
+            else:
+                control_image_t = (control_image_input_t + 1) / 2
 
-                if control_image.shape[1] != 3:
-                    raise RuntimeError("Control image must have 3 channels")
+            control_image_disp = control_image_t * 2 - 1
+            result_images["control"] = control_image_disp[:, [2, 0, 1], :, :]
+            log_img(control_image_disp, "control_image")
 
-                if control_image.min() < 0 or control_image.max() > 1:
-                    raise RuntimeError(
-                        f"Control image must be in [0, 1] but we received {control_image.min()} and {control_image.max()}"
-                    )
+            if len(control_image_t.shape) == 3:
+                raise RuntimeError("Control image must be 4D")
 
-                if control_image.max() <= 0.5:
-                    raise RuntimeError("Control image must be in [0, 1]")
+            if control_image_t.shape[1] != 3:
+                raise RuntimeError("Control image must have 3 channels")
 
-                if control_image.min() >= 0.5:
-                    raise RuntimeError("Control image must be in [0, 1]")
-
-                control_image_disp = control_image * 2 - 1
-                result_images["control"] = control_image_disp[:, [2, 0, 1], :, :]
-                log_img(control_image_disp, "control_image")
-                c_cat.append(control_image)
-
-            elif hasattr(model, "masked_image_key"):
-                # inpainting model
-                mask_t = pillow_img_to_torch_image(ImageOps.invert(mask_image_orig)).to(
-                    get_device()
+            if control_image_t.min() < 0 or control_image_t.max() > 1:
+                raise RuntimeError(
+                    f"Control image must be in [0, 1] but we received {control_image_t.min()} and {control_image_t.max()}"
                 )
-                inverted_mask = 1 - mask_latent
-                masked_image_t = init_image_t * (mask_t < 0.5)
-                log_img(masked_image_t, "masked_image")
 
-                inverted_mask_latent = torch.nn.functional.interpolate(
-                    inverted_mask, size=shape[-2:]
-                )
-                c_cat.append(inverted_mask_latent)
+            if control_image_t.max() == control_image_t.min():
+                raise RuntimeError("No control signal found in control image.")
 
-                masked_image_latent = model.get_first_stage_encoding(
-                    model.encode_first_stage(masked_image_t)
-                )
-                c_cat.append(masked_image_latent)
+            c_cat.append(control_image_t)
 
-            elif model.cond_stage_key == "edit":
-                # pix2pix model
-                c_cat = [model.encode_first_stage(init_image_t)]
-                c_cat_neutral = [torch.zeros_like(init_latent)]
-                denoiser_cls = CFGEditingDenoiser
-            if c_cat:
-                c_cat = [torch.cat(c_cat, dim=1)]
+        elif hasattr(model, "masked_image_key"):
+            # inpainting model
+            mask_t = pillow_img_to_torch_image(ImageOps.invert(mask_image_orig)).to(
+                get_device()
+            )
+            inverted_mask = 1 - mask_latent
+            masked_image_t = init_image_t * (mask_t < 0.5)
+            log_img(masked_image_t, "masked_image")
+
+            inverted_mask_latent = torch.nn.functional.interpolate(
+                inverted_mask, size=shape[-2:]
+            )
+            c_cat.append(inverted_mask_latent)
+
+            masked_image_latent = model.get_first_stage_encoding(
+                model.encode_first_stage(masked_image_t)
+            )
+            c_cat.append(masked_image_latent)
+
+        elif model.cond_stage_key == "edit":
+            # pix2pix model
+            c_cat = [model.encode_first_stage(init_image_t)]
+            c_cat_neutral = [torch.zeros_like(init_latent)]
+            denoiser_cls = CFGEditingDenoiser
+        if c_cat:
+            c_cat = [torch.cat(c_cat, dim=1)]
 
         if c_cat_neutral is None:
             c_cat_neutral = c_cat
@@ -487,52 +500,49 @@ def _generate_single_image(
             "c_concat": c_cat_neutral,
             "c_crossattn": [neutral_conditioning],
         }
-        log_latent(init_latent_noised, "init_latent_noised")
 
         if (
             prompt.allow_compose_phase
             and not is_controlnet_model
             and not model.cond_stage_key == "edit"
         ):
-            comp_samples = _generate_composition_latent(
-                sampler=sampler,
-                sampler_kwargs={
-                    "num_steps": prompt.steps,
-                    "initial_latent": init_latent_noised,
-                    "positive_conditioning": positive_conditioning,
-                    "neutral_conditioning": neutral_conditioning,
-                    "guidance_scale": prompt.prompt_strength,
-                    "t_start": t_enc,
-                    "mask": mask_latent,
-                    "orig_latent": init_latent,
-                    "shape": shape,
-                    "batch_size": 1,
-                    "denoiser_cls": denoiser_cls,
-                },
-            )
-            if comp_samples is not None:
-                result_images["composition"] = comp_samples
-                noise = noise[:, :, : comp_samples.shape[2], : comp_samples.shape[3]]
-
-                schedule = NoiseSchedule(
-                    model_num_timesteps=model.num_timesteps,
-                    ddim_num_steps=prompt.steps,
-                    model_alphas_cumprod=model.alphas_cumprod,
-                    ddim_discretize="uniform",
+            if prompt.init_image:
+                comp_image = _generate_composition_image(
+                    prompt=prompt,
+                    target_height=init_image.height,
+                    target_width=init_image.width,
+                    model=model,
+                    nsfw_filter=nsfw_filter,
+                    safety_feature_extractor=safety_feature_extractor,
+                    safety_checker=safety_checker,
+                    codeformer_model=codeformer_model,
+                    cutoff=get_model_default_image_size(prompt.model),
                 )
-                t_enc = int(prompt.steps * 0.75)
-                init_latent_noised = noise_an_image(
-                    comp_samples,
-                    torch.tensor([t_enc - 1]).to(get_device()),
-                    schedule=schedule,
-                    noise=noise,
+            else:
+                comp_image = _generate_composition_image(
+                    prompt=prompt,
+                    target_height=prompt.height,
+                    target_width=prompt.width,
+                    model=model,
+                    nsfw_filter=nsfw_filter,
+                    safety_feature_extractor=safety_feature_extractor,
+                    safety_checker=safety_checker,
+                    codeformer_model=codeformer_model,
+                    cutoff=get_model_default_image_size(prompt.model),
+                )
+            if comp_image is not None:
+                result_images["composition"] = comp_image
+                # noise = noise[:, :, : comp_image.height, : comp_image.shape[3]]
+                t_enc = int(prompt.steps * 0.65)
+                log_img(comp_image, "comp_image")
+                comp_image_t = pillow_img_to_torch_image(comp_image)
+                comp_image_t = comp_image_t.to(get_device())
+                init_latent = model.get_first_stage_encoding(
+                    model.encode_first_stage(comp_image_t)
                 )
-
-                log_latent(comp_samples, "comp_samples")
         with lc.timing("sampling"):
             samples = sampler.sample(
                 num_steps=prompt.steps,
-                initial_latent=init_latent_noised,
                 positive_conditioning=positive_conditioning,
                 neutral_conditioning=neutral_conditioning,
                 guidance_scale=prompt.prompt_strength,
@@ -542,6 +552,7 @@ def _generate_single_image(
                 shape=shape,
                 batch_size=1,
                 denoiser_cls=denoiser_cls,
+                noise=noise,
             )
         if return_latent:
             return samples
@@ -580,28 +591,29 @@ def _generate_single_image(
         if nsfw_filter and safety_score.is_filtered:
             progress_latents.clear()
         if not nsfw_filter or not safety_score.is_filtered:
-            if prompt.fix_faces and codformer_model:
+            if prompt.fix_faces and codeformer_model:
                 logger.info("Fixing 😊 's in 🖼  using CodeFormer...")
                 with lc.timing("face enhancement"):
                     gen_img = enhance_faces(gen_img,
-                                        net=codformer_model,
+                                        net=codeformer_model,
                                         fidelity=prompt.fix_faces_fidelity)
             if prompt.upscale:
                 logger.info("Upscaling 🖼  using real-ESRGAN...")
                 with lc.timing("upscaling"):
                     upscaled_img = upscale_image(gen_img)
 
-        # put the newly generated patch back into the original, full-size image
-        if prompt.mask_modify_original and mask_image_orig and starting_image:
-            img_to_add_back_to_original = upscaled_img if upscaled_img else gen_img
-            rebuilt_orig_img = combine_image(
-                original_img=starting_image,
-                generated_img=img_to_add_back_to_original,
-                mask_img=mask_image_orig,
-            )
+            # put the newly generated patch back into the original, full-size image
+            if prompt.mask_modify_original and mask_image_orig and starting_image:
+                img_to_add_back_to_original = upscaled_img if upscaled_img else gen_img
+                rebuilt_orig_img = combine_image(
+                    original_img=starting_image,
+                    generated_img=img_to_add_back_to_original,
+                    mask_img=mask_image_orig,
+                )
 
-        if prompt.caption_text:
-            add_caption_to_image(gen_img, prompt.caption_text)
+            if prompt.caption_text:
+                caption_text = prompt.caption_text.format(prompt=prompt.prompt_text)
+                add_caption_to_image(gen_img, caption_text)
 
         result = ImagineResult(
             img=gen_img,
@@ -616,6 +628,7 @@ def _generate_single_image(
             timings=lc.get_timings(),
             progress_latents=progress_latents.copy(),
         )
+
         _most_recent_result = result
         logger.info(f"Image Generated. Timings: {result.timings_str()}")
         return result
@@ -644,68 +657,77 @@ def calc_scale_to_fit_within(
     return max_size / height
 
 
-def _generate_composition_latent(
-    sampler,
-    sampler_kwargs,
+def _scale_latent(
+    latent,
+    model,
+    h,
+    w,
 ):
-    from copy import deepcopy
-
     from torch.nn import functional as F
 
-    from classy_imaginary.enhancers.upscale_riverwing import upscale_latent
+    # convert to non-latent-space first
+    img = model.decode_first_stage(latent)
+    img = F.interpolate(img, size=(h, w), mode="bicubic", align_corners=False)
+    latent = model.get_first_stage_encoding(model.encode_first_stage(img))
+    return latent
 
-    b, c, h, w = orig_shape = sampler_kwargs["shape"]
-    max_compose_gen_size = 768
-    shrink_scale = calc_scale_to_fit_within(
-        height=h,
-        width=w,
-        max_size=int(math.ceil(max_compose_gen_size / 8)),
-    )
-    if shrink_scale >= 1:
+
+def _generate_composition_image(prompt,
+                                target_height,
+                                target_width,
+                                model,
+                                nsfw_filter: bool = False,
+                                safety_feature_extractor=None,
+                                safety_checker=None,
+                                codeformer_model=None,
+                                cutoff=512):
+    from copy import copy
+
+    from PIL import Image
+
+    if prompt.width <= cutoff and prompt.height <= cutoff:
         return None
 
-    new_kwargs = deepcopy(sampler_kwargs)
-
-    # shrink everything
-    new_shape = b, c, int(round(h * shrink_scale)), int(round(w * shrink_scale))
-    initial_latent = new_kwargs["initial_latent"]
-    if initial_latent is not None:
-        initial_latent = F.interpolate(initial_latent, size=new_shape[2:], mode="area")
-
-    for cond in [
-        new_kwargs["positive_conditioning"],
-        new_kwargs["neutral_conditioning"],
-    ]:
-        cond["c_concat"] = [
-            F.interpolate(c, size=new_shape[2:], mode="area") for c in cond["c_concat"]
-        ]
-
-    mask_latent = new_kwargs["mask"]
-    if mask_latent is not None:
-        mask_latent = F.interpolate(mask_latent, size=new_shape[2:], mode="area")
-
-    orig_latent = new_kwargs["orig_latent"]
-    if orig_latent is not None:
-        orig_latent = F.interpolate(orig_latent, size=new_shape[2:], mode="area")
-    t_start = new_kwargs["t_start"]
-    if t_start is not None:
-        gen_strength = new_kwargs["t_start"] / new_kwargs["num_steps"]
-        t_start = int(round(15 * gen_strength))
-    new_kwargs.update(
-        {
-            "num_steps": 15,
-            "initial_latent": initial_latent,
-            "t_start": t_start,
-            "mask": mask_latent,
-            "orig_latent": orig_latent,
-            "shape": new_shape,
-        }
+    composition_prompt = copy(prompt)
+    shrink_scale = calc_scale_to_fit_within(
+        height=prompt.height,
+        width=prompt.width,
+        max_size=cutoff,
     )
-    samples = sampler.sample(**new_kwargs)
+    composition_prompt.width = int(prompt.width * shrink_scale)
+    composition_prompt.height = int(prompt.height * shrink_scale)
 
-    samples = upscale_latent(samples)
-    samples = F.interpolate(samples, size=orig_shape[2:], mode="bilinear")
-    return samples
+    composition_prompt.steps = None
+    composition_prompt.upscaled = False
+    composition_prompt.fix_faces = False
+    composition_prompt.mask_modify_original = False
+
+    composition_prompt.validate()
+
+    result = _generate_single_image(prompt=composition_prompt,
+                                    model=model,
+                                    nsfw_filter=nsfw_filter,
+                                    safety_feature_extractor=safety_feature_extractor,
+                                    safety_checker=safety_checker,
+                                    codeformer_model=codeformer_model)
+    img = result.images["generated"]
+    while img.width < target_width:
+        from classy_imaginary.enhancers.upscale_realesrgan import upscale_image
+
+        img = upscale_image(img)
+
+    # samples = _generate_single_image(composition_prompt, return_latent=True)
+    # while samples.shape[-1] * 8 < target_width:
+    #     samples = upscale_latent(samples)
+    #
+    # img = model_latent_to_pillow_img(samples)
+
+    img = img.resize(
+        (target_width, target_height),
+        resample=Image.Resampling.LANCZOS,
+    )
+
+    return img
 
 
 def prompt_normalized(prompt):
