@@ -25,6 +25,7 @@ SPLITMEM_ENABLED = False
 
 @dataclass
 class ModelConfig:
+    description: str
     short_name: str
     config_path: str
     weights_url: str
@@ -39,6 +40,7 @@ midas_url = "https://github.com/intel-isl/DPT/releases/download/1_0/dpt_hybrid-m
 
 MODEL_CONFIGS = [
     ModelConfig(
+        description="Stable Diffusion 1.4",
         short_name="SD-1.4",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/bstddev/sd-v1-4/resolve/77221977fa8de8ab8f36fac0374c120bd5b53287/sd-v1-4.ckpt",
@@ -46,6 +48,7 @@ MODEL_CONFIGS = [
         alias="sd14",
     ),
     ModelConfig(
+        description="Stable Diffusion 1.5",
         short_name="SD-1.5",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/runwayml/stable-diffusion-v1-5/resolve/889b629140e71758e1e0006e355c331a5744b4bf/v1-5-pruned-emaonly.ckpt",
@@ -54,6 +57,7 @@ MODEL_CONFIGS = [
         alias="sd15",
     ),
     ModelConfig(
+        description="Stable Diffusion 1.5 - Inpainting",
         short_name="SD-1.5-inpaint",
         config_path="configs/stable-diffusion-v1-inpaint.yaml",
         weights_url="https://huggingface.co/julienacquaviva/inpainting/resolve/2155ff7fe38b55f4c0d99c2f1ab9b561f8311ca7/sd-v1-5-inpainting.ckpt",
@@ -61,6 +65,7 @@ MODEL_CONFIGS = [
         alias="sd15in",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.0 - bad at making people",
         short_name="SD-2.0",
         config_path="configs/stable-diffusion-v2-inference.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2-base/resolve/main/512-base-ema.ckpt",
@@ -68,6 +73,7 @@ MODEL_CONFIGS = [
         alias="sd20",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.0 - Inpainting",
         short_name="SD-2.0-inpaint",
         config_path="configs/stable-diffusion-v2-inpainting-inference.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2-inpainting/resolve/main/512-inpainting-ema.ckpt",
@@ -75,6 +81,7 @@ MODEL_CONFIGS = [
         alias="sd20in",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.1",
         short_name="SD-2.1",
         config_path="configs/stable-diffusion-v2-inference.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2-1-base/resolve/main/v2-1_512-ema-pruned.ckpt",
@@ -82,6 +89,7 @@ MODEL_CONFIGS = [
         alias="sd21",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.1 - Inpainting",
         short_name="SD-2.1-inpaint",
         config_path="configs/stable-diffusion-v2-inpainting-inference.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2-inpainting/resolve/main/512-inpainting-ema.ckpt",
@@ -89,6 +97,7 @@ MODEL_CONFIGS = [
         alias="sd21in",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.1 v - 768x768",
         short_name="SD-2.1-v",
         config_path="configs/stable-diffusion-v2-inference-v.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2-1/resolve/main/v2-1_768-ema-pruned.ckpt",
@@ -96,6 +105,7 @@ MODEL_CONFIGS = [
         forced_attn_precision="fp32",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.0 v - 768x768",
         short_name="SD-2.0-v",
         config_path="configs/stable-diffusion-v2-inference-v.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2/resolve/main/768-v-ema.ckpt",
@@ -103,6 +113,7 @@ MODEL_CONFIGS = [
         alias="sd20v",
     ),
     ModelConfig(
+        description="Stable Diffusion 2.0 - Depth",
         short_name="SD-2.0-depth",
         config_path="configs/stable-diffusion-v2-midas-inference.yaml",
         weights_url="https://huggingface.co/stabilityai/stable-diffusion-2-depth/resolve/main/512-depth-ema.ckpt",
@@ -110,6 +121,7 @@ MODEL_CONFIGS = [
         alias="sd20dep",
     ),
     ModelConfig(
+        description="Instruct Pix2Pix - Photo Editing",
         short_name="instruct-pix2pix",
         config_path="configs/instruct-pix2pix.yaml",
         weights_url="https://huggingface.co/imaginairy/instruct-pix2pix/resolve/ea0009b3d0d4888f410a40bd06d69516d0b5a577/instruct-pix2pix-00-22000-pruned.ckpt",
@@ -119,6 +131,7 @@ MODEL_CONFIGS = [
     ),
     # --- Custom model --#
     ModelConfig(
+        description="OpenJourney V1",
         short_name="openjourney-v1",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/prompthero/openjourney/resolve/7428477dad893424c92f6ea1cc29d45f6d1448c1/mdjrny-v4.safetensors",
@@ -127,6 +140,7 @@ MODEL_CONFIGS = [
         alias="oj1",
     ),
     ModelConfig(
+        description="OpenJourney V2",
         short_name="openjourney-v2",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/prompthero/openjourney-v2/resolve/47257274a40e93dab7fbc0cd2cfd5f5704cfeb60/openjourney-v2.ckpt",
@@ -135,6 +149,7 @@ MODEL_CONFIGS = [
         alias="oj2",
     ),
     ModelConfig(
+        description="MidJourney PaperCut",
         short_name="MidJourney-PaperCut",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/ShadoWxShinigamI/MidJourney-PaperCut/resolve/main/Mdjrny-pprct_step_7000.ckpt",
@@ -142,6 +157,7 @@ MODEL_CONFIGS = [
         alias="mjpc"
     ),
     ModelConfig(
+        description="OpenJourney",
         short_name="Openjourney",
         config_path="configs/stable-diffusion-v1.yaml",
         # config_path="configs/stable-diffusion-v2-inference-v.yaml",
@@ -151,6 +167,7 @@ MODEL_CONFIGS = [
         alias="ojc"
     ),
     ModelConfig(
+        description="Protogen Anime",
         short_name="Protogen-Anime",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://civitai.com/api/download/models/4007",
@@ -158,6 +175,7 @@ MODEL_CONFIGS = [
         alias="proa"
     ),
     ModelConfig(
+        description="Protogen Photo",
         short_name="Protogen-Photo",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/darkstorm2150/Protogen_x3.4_Official_Release/resolve/main/ProtoGen_X3.4.ckpt",
@@ -165,6 +183,7 @@ MODEL_CONFIGS = [
         alias="prop"
     ),
     ModelConfig(
+        description="Protogen Photo V2",
         short_name="Protogen-PhotoV2",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/darkstorm2150/Protogen_v5.3_Official_Release/resolve/main/ProtoGen_X5.3.ckpt",
@@ -172,6 +191,7 @@ MODEL_CONFIGS = [
         alias="prop2"
     ),
     ModelConfig(
+        description="Role Playing Game",
         short_name="RPG",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/Anashel/rpg/resolve/main/RPG-V4-Model-Download/RPG-v4.safetensors",
@@ -179,30 +199,35 @@ MODEL_CONFIGS = [
         alias="rpg"
     ),
     ModelConfig(
+        description="Synthwave Punk",
         short_name="SynthwavePunk",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/zipp425/synthwavePunk/resolve/main/synthwavePunk_v2.ckpt",
         default_image_size=512,
     ),
     ModelConfig(
+        description="Modern Disney",
         short_name="ModernDisney",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/nitrosocke/mo-di-diffusion/resolve/main/moDi-v1-pruned.ckpt",
         default_image_size=512,
     ),
     ModelConfig(
+        description="Art Diffusion",
         short_name="Art-Diffusion",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/DGSpitzer/DGSpitzer-Art-Diffusion/resolve/main/DGSpitzer-Art-Diffusion.ckpt",
         default_image_size=512,
     ),
     ModelConfig(
+        description="Graphic Art",
         short_name="graphic-art",
         config_path="configs/graphicArt_graphicArtBeta.yaml",
         weights_url="https://civitai.com/api/download/models/9297?type=Model&format=PickleTensor",
         default_image_size=768,
     ),
     ModelConfig(
+        description="DreamLike Humans",
         short_name="DreamLikeHumans",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/dreamlike-art/dreamlike-photoreal-2.0/resolve/main/dreamlike-photoreal-2.0.ckpt",
@@ -210,6 +235,7 @@ MODEL_CONFIGS = [
         alias="dlh"
     ),
     ModelConfig(
+        description="DreamLike Diffusion",
         short_name="DreamLikeDiffusion",
         config_path="configs/stable-diffusion-v1.yaml",
         weights_url="https://huggingface.co/dreamlike-art/dreamlike-diffusion-1.0/resolve/main/dreamlike-diffusion-1.0.ckpt",
